


// Function to make a computer move based on the selected level
function makeComputerMove() {

  const legalMoves = getLegalMoves();
  const randomMove = legalMoves[Math.floor(Math.random() * legalMoves.length)];

  switch (computerLevel) {
    case "beginner":
      // Implement beginner level computer move logic here (e.g., random moves)
      // For example:
      // const legalMoves = getLegalMoves();
      // const randomMove = legalMoves[Math.floor(Math.random() * legalMoves.length)];
      makeMove(randomMove);
      break;
    case "normal":
      // Implement normal level computer move logic using minimax algorithm with a moderate depth
      // For example:
      // let normalMove = getBestMoveWithMinimax(3); // Use depth 3 for normal level

		makeMove(randomMove);

		// makeMove(normalMove);

      
      break;

    case "advanced":
      // Implement advanced level computer move logic using minimax algorithm with a higher depth
      // For example:
      // let advancedMove = getBestMoveWithMinimax(5); // Use depth 5 for advanced level

		makeMove(randomMove);

		// makeMove(advancedMove);


      break;
    default:
      console.error("Invalid computer level.");
      break;
  }
}

function getBestMoveWithMinimax(depth) {
  let bestMove = null;
  let bestScore = -99999;

  // Get all legal moves for the current player
  const legalMoves = getLegalMoves();

  // Loop through each legal move and evaluate its score using minimax
  for (const move of legalMoves) {
    // Make the move and get the score using minimax with alpha-beta pruning
    const score = minimax(move, depth, -99999, 99999, false);

    // If the score is better than the previous best, update the best move
    if (score > bestScore) {
      bestScore = score;
      bestMove = move;
    }

    // Undo the move to revert to the original state for the next iteration
    undoMove();
  }

  return bestMove;
}

function minimax(move, depth, alpha, beta, maximizingPlayer) {
  // Make the move on the board
  makeMoveEval(move);

  // If the maximum depth is reached or the game is over, evaluate the board state and return its score
  if (depth === 0 || isGameOver()) {
    const score = evaluateBoard();
    undoMove(); // Undo the move to revert to the original state
    return score;
  }

  if (maximizingPlayer) {
    let maxScore = -99999;
    const legalMoves = getLegalMoves();

    for (const move of legalMoves) {
      maxScore = Math.max(maxScore, minimax(move, depth - 1, alpha, beta, false));
      alpha = Math.max(alpha, maxScore);

      // Alpha-beta pruning
      if (alpha >= beta) {
        break;
      }
    }

    undoMove(); // Undo the move to revert to the original state
    return maxScore;
  } else {
    let minScore = 99999;
    const legalMoves = getLegalMoves();

    for (const move of legalMoves) {
      minScore = Math.min(minScore, minimax(move, depth - 1, alpha, beta, true));
      beta = Math.min(beta, minScore);

      // Alpha-beta pruning
      if (alpha >= beta) {
        break;
      }
    }

    undoMove(); // Undo the move to revert to the original state
    return minScore;
  }
}

function makeMoveEval(move) {
  const { from, to } = move;

  // Update the board to reflect the move
  board[to] = board[from];
  board[from] = null;

  // Check if the move results in a piece promotion (e.g., a pawn reaching the 8th rank)
  // if (isPromotionMove(move)) {
  //   handlePromotion(move);
  // }

  // Check if the move captures an opponent's piece
  const capturedPiece = board[to];
  if (capturedPiece) {
    // Add the captured piece to the move object (for undoMove() function)
    move.capturedPiece = capturedPiece;
  }

  // Toggle player turn
  // currentPlayer = 1 - currentPlayer;
	switchPlayer();

    // Switch turns
    // isWhiteTurn = !isWhiteTurn;


  // Save the move in the history array
  history.push(move);
}

function makeMove(move) {
  const { from, to } = move;

  // Update the board to reflect the move
  board[to] = board[from];
  board[from] = null;

	const sourceRow = Math.floor(from / 8);
	const sourceCol = from % 8;
	const clickedRow = Math.floor(to / 8);
	const clickedCol = to % 8;

  // Check if the move results in a piece promotion (e.g., a pawn reaching the 8th rank)
  // if (isPromotionMove(move)) {
  //   handlePromotion(move);
  // }

  // Check if the move captures an opponent's piece
  const capturedPiece = board[to];

  if (capturedPiece) {
    // Add the captured piece to the move object (for undoMove() function)
    move.capturedPiece = capturedPiece;

  }


	let selectedPieceX = findPiece(sourceCol, sourceRow);

	const pieceInd = selectedPieceX.ind;

	movePiece(selectedPieceX, clickedCol, clickedRow);

	// blackPieces[pieceInd].x = clickedCol;
	// blackPieces[pieceInd].y = clickedRow;

  // Toggle player turn
  // currentPlayer = 1 - currentPlayer;
	switchPlayer();

    // Switch turns
    isWhiteTurn = !isWhiteTurn;


  // Save the move in the history array
  history.push(move);

	selectedPieceX = null;

    drawPieces();
	printMoves();

  // Highlight the clicked square

  const clickedCol1 = isBoardFlipped ? 7 - clickedCol : clickedCol;
  const clickedRow1 = isBoardFlipped ? 7 - clickedRow : clickedRow;

  ctx.fillStyle = "rgba(255, 255, 0, 0.5)";
  ctx.fillRect(clickedCol1 * squareSize, clickedRow1 * squareSize, squareSize, squareSize);


}


function undoMove() {
  if (history.length > 0) {
    const lastMove = history.pop();
    const { from, to, capturedPiece } = lastMove;

    // Move the piece back to its original position
    board[to] = board[from];
    board[from] = null;

    // If there was a captured piece, restore it to the captured position
    if (capturedPiece) {
      board[to] = capturedPiece;
    }

    // Toggle player turn
    // currentPlayer = 1 - currentPlayer;
	switchPlayer();
	isWhiteTurn = !isWhiteTurn;
  }
}

function getLegalMoves() {
  const legalMoves = [];

  for (let from = 0; from < 64; from++) {
    const piece = board[from];
    if (piece && piece.color === currentPlayer) {
      switch (piece.type) {
        case "pawn":
          addPawnMoves(legalMoves, from);
          break;
        case "knight":
          addKnightMoves(legalMoves, from);
          break;
        case "bishop":
          addBishopMoves(legalMoves, from);
          break;



        // Add cases for other piece types (e.g., rook, bishop, queen, king)
      }
    }
  }

  return legalMoves;
}

function addPawnMoves(legalMoves, from) {
  const piece = board[from];
  const color = piece.color;
  const forwardDirection = color === "white" ? -1 : 1;
  const startRank = color === "white" ? 6 : 1;
  const promotionRank = color === "white" ? 0 : 7;

  // Move one square forward
  const toOneStep = from + 8 * forwardDirection;
  if (board[toOneStep] === null) {
    // Normal move to an empty square
    legalMoves.push({ from, to: toOneStep });

    // Move two squares forward (allowed only from the starting rank)
    if (from >= 8 * startRank && board[toOneStep + 8 * forwardDirection] === null) {
      legalMoves.push({ from, to: toOneStep + 8 * forwardDirection });
    }
  }

  // Capture diagonally to the left
  const toDiagonalLeft = from + 8 * forwardDirection - 1;
  if (isValidCaptureMove(from, toDiagonalLeft)) {
    legalMoves.push({ from, to: toDiagonalLeft });
  }

  // Capture diagonally to the right
  const toDiagonalRight = from + 8 * forwardDirection + 1;
  if (isValidCaptureMove(from, toDiagonalRight)) {
    legalMoves.push({ from, to: toDiagonalRight });
  }

  // Check for pawn promotion (reaching the promotion rank)
  if (toOneStep >= 8 * promotionRank && toOneStep <= 8 * promotionRank + 7) {
    // Promote to queen for simplicity (you can customize this for other pieces)
    legalMoves.forEach(move => (move.promotion = "queen"));
  }
}

function isValidCaptureMove(from, to) {
  // Check if the destination square is within the board boundaries and contains an opponent's piece
  if (to >= 0 && to <= 63) {
    const pieceFrom = board[from];
    const pieceTo = board[to];
    return pieceTo && pieceTo.color !== pieceFrom.color;
  }
  return false;
}


function addKnightMoves(legalMoves, from) {
  const knightMoves = [-17, -15, -10, -6, 6, 10, 15, 17]; // Relative knight moves

  for (const moveOffset of knightMoves) {
    const to = from + moveOffset;

    // Check if the destination square is within the board boundaries
    if (to >= 0 && to < 64) {
      const fromCol = from % 8;
      const fromRow = Math.floor(from / 8);
      const toCol = to % 8;
      const toRow = Math.floor(to / 8);

      // Check if the knight's move is valid (within an L-shape pattern)
      if (Math.abs(fromCol - toCol) === 1 && Math.abs(fromRow - toRow) === 2) {
        addMoveIfValid(legalMoves, from, to);
      } else if (Math.abs(fromCol - toCol) === 2 && Math.abs(fromRow - toRow) === 1) {
        addMoveIfValid(legalMoves, from, to);
      }
    }
  }
}

function addMoveIfValid(legalMoves, from, to) {
  const pieceFrom = board[from];
  const pieceTo = board[to];

  // Check if the destination square is empty or contains an opponent's piece
  if (!pieceTo || pieceTo.color !== pieceFrom.color) {
    legalMoves.push({ from, to });
  }
}



function isValidMove(from, to, color) {
  // Check if the destination square is within the board boundaries
  if (to < 0 || to >= 64) {
    return false;
  }

  const pieceFrom = board[from];
  const pieceTo = board[to];

  // Check if the destination square is empty or contains an opponent's piece
  if (!pieceTo || pieceTo.color !== color) {
    return true;
  }

  return false;
}

function addBishopMoves(legalMoves, from) {
  const bishopMoves = [-9, -7, 7, 9]; // Relative bishop moves

  for (const moveOffset of bishopMoves) {
    let to = from + moveOffset;

    // Check if the destination square is within the board boundaries
    while (to >= 0 && to < 64) {
      const fromCol = from % 8;
      const fromRow = Math.floor(from / 8);
      const toCol = to % 8;
      const toRow = Math.floor(to / 8);

      // Check if the bishop's move is diagonal (same distance in both rows and columns)
      if (Math.abs(fromCol - toCol) === Math.abs(fromRow - toRow)) {
        addMoveIfValid(legalMoves, from, to);

        // Stop exploring in this direction if there's a piece on the destination square
        if (board[to]) {
          break;
        }
      } else {
        break; // Stop exploring in this direction if the move is not diagonal
      }

      // Move to the next square in the same direction
      to += moveOffset;
    }
  }
}







function evaluateBoard() {
  let score = 0;

  // Loop through the entire board
  for (let square = 0; square < 64; square++) {
    const piece = board[square];

    // If the square is empty, continue to the next square
    if (!piece) continue;

    // Determine the value of each piece type (adjust these values to customize the AI's behavior)
    let pieceValue = 0;
    switch (piece.type) {
      case "pawn":
        pieceValue = 10;
        break;
      case "knight":
        pieceValue = 30;
        break;
      case "bishop":
        pieceValue = 30;
        break;
      case "rook":
        pieceValue = 50;
        break;
      case "queen":
        pieceValue = 90;
        break;
      case "king":
        pieceValue = 900;
        break;
    }

    // Add the piece value to the score (positive value for white, negative for black)
    score += piece.color === "white" ? pieceValue : -pieceValue;
  }

  return score;
}


function isGameOver() {
  // Check for checkmate or stalemate (assuming you have implemented these functions)
  // if (isCheckmate(currentPlayer) || isStalemate(currentPlayer)) {
  //  return true;
  //}

  // Check for insufficient material (e.g., king vs. king or king vs. king + minor piece)
  // if (isInsufficientMaterial()) {
  //  return true;
  //}

  // Add other game-over conditions if necessary

  return false; // If none of the game-over conditions are met, return false
}


// Function to start the game
function startGame() {
  // Your code to initialize the chessboard and game logic goes here
  // ...
  // Example: initializeBoard();

    // initializeBoard();
	location.reload(); // Reload the page

}

// Add an event listener for the "Start" button to begin the game and get the selected computer level
document.getElementById("start-button").addEventListener("click", () => {
  getComputerLevel();
  startGame();
});