
canvas.addEventListener("click", handleCanvasClick);

selectedPiece = null;
arrayIndex = -1;

 function findPiece(col, row) {
  // Search for the piece in the whitePieces array
  for (let i = 0; i < whitePieces.length; i++) {
    const piece = whitePieces[i];
    if (piece.x === col && piece.y === row && piece.active) {
	  if (selectedPiece === null) {arrayIndex = i;}
      return piece;
    }
  }

  // Search for the piece in the blackPieces array
  for (let i = 0; i < blackPieces.length; i++) {
    const piece = blackPieces[i];
    if (piece.x === col && piece.y === row && piece.active) {
	  if (selectedPiece === null) {arrayIndex = i;}
      return piece;
    }
  }

  // No piece found at the specified column and row

  return null;
 }


function capturePiece(piece) {
  // Check if the captured piece is valid and active
  if (!piece || !piece.active) {
    return;
  }

  // Update the captured piece's active status
  piece.active = false;

  // Remove the captured piece from the respective array
  if (isPieceWhite(piece)) {
    const index = whitePieces.indexOf(piece);
    if (index !== -1) {
      // whitePieces.splice(index, 1);
    }
  } else {
    const index = blackPieces.indexOf(piece);
    if (index !== -1) {
      // blackPieces.splice(index, 1);
    }
  }

  // Print capture information to the console
  console.log(`Captured ${piece.name}`);
}

function isPieceWhite(piece) {
  return piece.name.charAt(0) === "w";
}




function movePiece(selectedPiece, clickedCol, clickedRow) {

  // Check if the selected piece is valid and active
  if (!selectedPiece || !selectedPiece.active) {
    return;
  }


  // Check if there is a piece at the destination location
  const destinationPiece = findPiece(clickedCol, clickedRow);

  if (destinationPiece !== null) {
    // Handle capturing the opponent's piece
    if (isPieceWhite(selectedPiece) !== isPieceWhite(destinationPiece)) {
      capturePiece(destinationPiece);
    } else {
      // Can't capture own piece
      return;
    }
  }

  // Handle castling if the selected piece is a king
  if (selectedPiece.name.charAt(1) === "K") {
    const dx = Math.abs(selectedPiece.x - clickedCol);

    // Check if it's a valid castling move
    if (dx === 2 && clickedRow === selectedPiece.y && !selectedPiece.hasMoved) {
      let rook;
      let rookFromCol;
      let rookToCol;

      // Determine the rook's position based on the castling direction
      if (clickedCol < selectedPiece.x) {
        // Queen-side castling
        rook = findPiece(0, clickedRow);
        rookFromCol = 0;
        rookToCol = clickedCol + 1;
      } else {
        // King-side castling
        rook = findPiece(7, clickedRow);
        rookFromCol = 7;
        rookToCol = clickedCol - 1;
      }

      // Check if the rook exists and has not moved
      if (rook && !rook.hasMoved) {
        // Check if there are no obstructions between the king and rook
        const minCol = Math.min(selectedPiece.x, rookFromCol) + 1;
        const maxCol = Math.max(selectedPiece.x, rookFromCol);
        for (let col = minCol; col < maxCol; col++) {
          if (findPiece(col, clickedRow) !== null) {
            // Obstruction found, castling is not allowed
            return;
          }
        }

        // Move the rook to its new position
        rook.x = rookToCol;
        rook.hasMoved = true;

        // Update the piece's coordinates and hasMoved flag
        selectedPiece.x = clickedCol;
        selectedPiece.hasMoved = true;

        // Print the castling move information to the console
        console.log(`Performed castling: ${selectedPiece.name} to`);

        return;
      }
    }
  }

  // Regular move


  // Update the piece's coordinates and hasMoved flag
  selectedPiece.x = clickedCol;
  selectedPiece.y = clickedRow;
  selectedPiece.hasMoved = true;

  // Print move information to the console
  console.log('Moved ${selectedPiece.name} to ');

}

// xxxxxxxxxxxx   Common functions above 



function handleCanvasClick(event) {

  // squareSize = canvas.width / 8;

  const rect = canvas.getBoundingClientRect();
  const x = event.clientX - rect.left;
  const y = event.clientY - rect.top;

  const clickedCol1 = Math.floor(x / squareSize);
  const clickedRow1 = Math.floor(y / squareSize);

  const clickedCol = isBoardFlipped ? 7 - clickedCol1 : clickedCol1;
  const clickedRow = isBoardFlipped ? 7 - clickedRow1 : clickedRow1;

  // console.log("left:", rect.left, "top:", rect.top);
  // console.log("cX:", event.clientX, "cy:", event.clientY);
  console.log("x:", clickedCol, "y:", clickedRow);

  // Remove highlight from previously clicked square
  // ctx.clearRect(0, 0, canvas.width, canvas.height);
  // drawPieces();

  // squareSize = canvas.width / 8;




function getPieceIndex(piece) {
  if (isPieceWhite(piece)) {
    return whitePieces.indexOf(piece);
  } else {
    return blackPieces.indexOf(piece);
  }
}

function canMovePiece(piece, toCol, toRow) {
  // Implement the movement rules for each type of piece
  const fromCol = piece.x;
  const fromRow = piece.y;

  const dx = Math.abs(fromCol - toCol);
  const dy = Math.abs(fromRow - toRow);

  // Check if there is a piece at the destination location
  const destinationPiece = findPiece(toCol, toRow);

  // Prevent capturing own piece
  if (destinationPiece !== null && isPieceWhite(destinationPiece) === isPieceWhite(piece)) {
    return false;
  }

  switch (piece.name.charAt(1)) {
    case "p": // Pawn
      if (isPieceWhite(piece)) {
        // White pawn
        if (fromCol === toCol && toRow === fromRow - 1) {
          // Move one square forward
          return true;
        } else if (
          fromCol === toCol &&
          fromRow === 6 &&
          toRow === 4 &&
          findPiece(toCol, toRow) === null &&
          findPiece(toCol, toRow + 1) === null
        ) {
          // Move two squares forward on the first move
          return true;
        } else if (
          Math.abs(fromCol - toCol) === 1 &&
          fromRow - toRow === 1 &&
          findPiece(toCol, toRow) !== null &&
          !isPieceWhite(findPiece(toCol, toRow))
        ) {
          // Capture diagonally
          return true;
        }
      } else {
        // Black pawn
        if (fromCol === toCol && toRow === fromRow + 1) {
          // Move one square forward
          return true;
        } else if (
          fromCol === toCol &&
          fromRow === 1 &&
          toRow === 3 &&
          findPiece(toCol, toRow) === null &&
          findPiece(toCol, toRow - 1) === null
        ) {
          // Move two squares forward on the first move
          return true;
        } else if (
          Math.abs(fromCol - toCol) === 1 &&
          toRow - fromRow === 1 &&
          findPiece(toCol, toRow) !== null &&
          isPieceWhite(findPiece(toCol, toRow))
        ) {
          // Capture diagonally
          return true;
        }
      }
      break;

    case "R": // Rook
      if (fromCol === toCol && fromRow !== toRow) {
        // Move vertically
        const minRow = Math.min(fromRow, toRow) + 1;
        const maxRow = Math.max(fromRow, toRow);
        for (let row = minRow; row < maxRow; row++) {
          if (findPiece(fromCol, row) !== null) {
            return false; // Cannot jump over pieces
          }
        }
        return true;
      } else if (fromRow === toRow && fromCol !== toCol) {
        // Move horizontally
        const minCol = Math.min(fromCol, toCol) + 1;
        const maxCol = Math.max(fromCol, toCol);
        for (let col = minCol; col < maxCol; col++) {
          if (findPiece(col, fromRow) !== null) {
            return false; // Cannot jump over pieces
          }
        }
        return true;
      }
      break;

    case "N": // Knight
      if ((dx === 1 && dy === 2) || (dx === 2 && dy === 1)) {
        return true;
      }
      break;

    case "B": // Bishop
      if (dx === dy) {
        // Move diagonally
        const colDir = toCol > fromCol ? 1 : -1;
        const rowDir = toRow > fromRow ? 1 : -1;
        let col = fromCol + colDir;
        let row = fromRow + rowDir;
        while (col !== toCol) {
          if (findPiece(col, row) !== null) {
            return false; // Cannot jump over pieces
          }
          col += colDir;
          row += rowDir;
        }
        return true;
      }
      break;

    case "Q": // Queen
      if (fromCol === toCol && fromRow !== toRow) {
        // Move vertically
        const minRow = Math.min(fromRow, toRow) + 1;
        const maxRow = Math.max(fromRow, toRow);
        for (let row = minRow; row < maxRow; row++) {
          if (findPiece(fromCol, row) !== null) {
            return false; // Cannot jump over pieces
          }
        }
        return true;
      } else if (fromRow === toRow && fromCol !== toCol) {
        // Move horizontally
        const minCol = Math.min(fromCol, toCol) + 1;
        const maxCol = Math.max(fromCol, toCol);
        for (let col = minCol; col < maxCol; col++) {
          if (findPiece(col, fromRow) !== null) {
            return false; // Cannot jump over pieces
          }
        }
        return true;
      } else if (dx === dy) {
        // Move diagonally
        const colDir = toCol > fromCol ? 1 : -1;
        const rowDir = toRow > fromRow ? 1 : -1;
        let col = fromCol + colDir;
        let row = fromRow + rowDir;
        while (col !== toCol) {
          if (findPiece(col, row) !== null) {
            return false; // Cannot jump over pieces
          }
          col += colDir;
          row += rowDir;
        }
        return true;
      }
      break;

    case "K": // King
      if (dx <= 1 && dy <= 1) {
        return true; // Standard king movement
      } else if (dx === 2 && dy === 0 && piece.hasMoved === false) {
        // Check for castling
        if (toCol === 6 && toRow === fromRow && !isSquareAttacked(fromCol + 1, fromRow, !isPieceWhite(piece)) && !isSquareAttacked(fromCol + 2, fromRow, !isPieceWhite(piece))) {
          // Short castle
          const rook = findPiece(7, fromRow);
          if (rook && rook.hasMoved === false) {
            return true;
          }
        } else if (toCol === 2 && toRow === fromRow && !isSquareAttacked(fromCol - 1, fromRow, !isPieceWhite(piece)) && !isSquareAttacked(fromCol - 2, fromRow, !isPieceWhite(piece))) {
          // Long castle
          const rook = findPiece(0, fromRow);
          if (rook && rook.hasMoved === false) {
            return true;
          }
        }
      }
      break;
  }

  return false; // Invalid move
}





function isSquareAttacked(col, row, isAttackerWhite) {
  for (let i = 0; i < whitePieces.length; i++) {
    const piece = whitePieces[i];

    if (isAttackerWhite === true && piece.active === true && canMovePiece(piece, col, row)) {
      return true;
    }
  }

  for (let i = 0; i < blackPieces.length; i++) {
    const piece = blackPieces[i];

    if (isAttackerWhite === false && piece.active === true && canMovePiece(piece, col, row)) {
      return true;
    }
  }

  return false;
}



// canvas main code:

if (selectedPiece === null) {
  // No piece is currently selected, so check if the clicked square contains a valid piece
  const piece = findPiece(clickedCol, clickedRow);

  if (piece !== null) {
    // Check if it is the correct player's turn
    if ((isWhiteTurn && !isPieceWhite(piece)) || (!isWhiteTurn && isPieceWhite(piece))) {
      // Display a warning message
      alert("It's not your turn to move that piece!");
      return;
    }

    // Select the clicked piece
    console.log("Clicked Piece:", piece.name);
    selectedPiece = piece;
    selectedPieceIndex = getPieceIndex(piece);

  // Store the source column and row
  sourceCol = clickedCol;
  sourceRow = clickedRow;

  // Highlight the clicked square
  ctx.fillStyle = "rgba(255, 255, 0, 0.5)";
  ctx.fillRect(clickedCol1 * squareSize, clickedRow1 * squareSize, squareSize, squareSize);


  }
} else {
  // A piece is already selected, so try to move it to the clicked square
  console.log("Clicked Piece:", selectedPiece.name, "Moving it now!");
  const isValidMove = canMovePiece(selectedPiece, clickedCol, clickedRow);

  if (isValidMove) {
    movePiece(selectedPiece, clickedCol, clickedRow);

	// updateMoveList(move);
  const moveTxt = {
    piece: selectedPiece.name,
    from: { col: sourceCol, row: sourceRow },
    to: { col: clickedCol, row: clickedRow }
  };

  if (isWhiteTurn) {
    whMove.push(moveTxt);
  } else {
    blMove.push(moveTxt);
  }

    // Switch turns
    isWhiteTurn = !isWhiteTurn;
	switchPlayer();

	// Update the board to reflect the move
	const toX = clickedRow * 8 + clickedCol;
	const fromX = sourceRow * 8 + sourceCol;
	board[toX] = board[fromX];
	board[fromX] = null;

    // Clear the selected piece and redraw the board
    selectedPiece = null;
    // drawPieces();
	// printMoves();

	makeComputerMove();

  }

  // Clear the selection regardless of the move's validity
  selectedPiece = null;
  selectedPieceIndex = -1;
}




} // canvas listener


// Example usage
/*
const piece = { name: "wP", x: 4, y: 6 }; // White pawn at column 4, row 6
const toCol = 4;
const toRow = 4;

if (canMovePiece(piece, toCol, toRow)) {
  // Move the piece to the new position
  piece.x = toCol;
  piece.y = toRow;
  console.log("Piece moved successfully!");
} else {
  console.log("Invalid move!");
}
*/

